from distutils.core import setup
from distutils.extension import Extension
from Cython.Build import cythonize
import numpy

extensions = [
    Extension(
        "special_partition",
        ["special_partition.pyx"],
        include_dirs=[numpy.get_include()],
        define_macros=[("NPY_NO_DEPRECATED_API", "NPY_1_7_API_VERSION")],
    )
]

setup(ext_modules=cythonize(extensions))

"""
Build instructions: 
------------------
> cd special_partition
> python setup.py build_ext --inplace
"""



import ujson
import os

from bioel.utils.bigbio_utils import load_bigbio_dataset
from tqdm import tqdm
from collections import defaultdict


def extract_document_text(output_dir, all_dataset):
    """
    Collect and consolidate textual data from various medical datasets
    ---------
    Parameter
    - output_dir : Path to directory where to save "abbreviations.json" file
    - all_dataset : Datasets for which you want to find abbreviations
    """

    # Make sure the output directory exists, create it if it doesn't
    os.makedirs(output_dir, exist_ok=True)

    output_file = os.path.join(output_dir, f"all_article_text.txt")

    # Make sure we don't duplicate anything
    all_pmids = set([])
    all_full_text = defaultdict(set)
    total_docs = 0

    # Loop through all datasets
    for dataset in tqdm(all_dataset):
        data = load_bigbio_dataset(dataset)
        for split in data.keys():
            for doc in data[split]:
                pmid = doc["document_id"]
                if pmid in all_pmids:
                    continue

                all_pmids.add(pmid)
                doc_text = " ".join([" ".join(p["text"]) for p in doc["passages"]])
                all_full_text[pmid].add(doc_text)

    # # PlantNorm
    # print("Running Plant Norm")
    # for subset in ['training','test','development']:
    #     with open(f'/Users/david/Downloads/DMCB_plant_{subset}_corpus.txt', 'r', encoding='utf-8', errors='ignore') as g:
    #         all_text = g.read()
    #         abstracts = all_text.strip().split('\n\n')
    #         abstract_lines = [x.split('\n') for x in abstracts]
    #         for abs in tqdm(abstract_lines):
    #             pmid = abs[0].split('|')[0]
    #             if pmid in all_pmids:
    #                 continue
    #             if len(abs[0].split('|')) == 1:
    #                 abs.pop(0)
    #             title = abs[0].split('|')[1]
    #             abs_text = abs[1].split('|')[1]
    #             doc_text = ' '.join([title, abs_text])

    #             all_pmids.add(pmid)
    #             all_full_text[pmid].add(doc_text)

    # Remove newlines that will interfere with Ab3P
    for pmid, text_set in all_full_text.items():
        text = list(text_set)[0]
        if "\n" in text:
            print(pmid)
        all_full_text[pmid] = text.replace("\n", " ")

    # Write output to file
    with open(output_file, "w") as f:
        output = "\n\n".join(
            [pmid + " | " + doc_text for pmid, doc_text in all_full_text.items()]
        )
        f.write(output)


def process_abbreviations(output_dir, all_dataset):
    """
    Process a list of abbreviations from a text file,
    filtering and collecting them into a JSON format based on a confidence score,
    and then saving the result to a specified output directory
    ---------
    Parameter
    - output_dir : Path to directory where to save "abbreviations.json" file
    - all_dataset : Datasets for which you want to find abbreviations
    """
    # Set up necessary variables/parameters
    all_abbreviations = {}
    min_confidence_cutoff = 0.95
    omitted = 0
    included = 0

    # Make sure the output directory exists
    os.makedirs(output_dir, exist_ok=True)

    output_file = os.path.join(output_dir, "abbreviations.json")

    input_file = os.path.join(output_dir, "raw_abbreviations.txt")

    with open(input_file, "r") as f:
        chunks = f.read().strip().split("\n\n")

    # Get abbreviations from each article
    for chunk in chunks:
        lines = chunk.split("\n")
        pmid = lines[0].split("|")[0].strip()
        abbrev_dict = {}
        for line in lines[1:]:
            abbrev, long_form, confidence_score = line.strip().split("|")
            confidence_score = float(confidence_score)
            if confidence_score > min_confidence_cutoff:
                abbrev_dict[abbrev] = long_form
                included += 1
            else:
                # print(abbrev, long_form, confidence_score)
                omitted += 1

        all_abbreviations[pmid] = abbrev_dict

    with open(output_file, "w") as f:
        f.write(ujson.dumps(all_abbreviations, indent=2))



import os
import ujson
import warnings
from collections import defaultdict
import pandas as pd
#from bigbio.dataloader import BigBioConfigHelpers
import json
import joblib
import numpy as np
import datetime

from tqdm.auto import tqdm

tqdm.pandas()

from datasets import load_dataset
from bioel.utils.bigbio_utils import dataset_to_documents, dataset_to_df, resolve_abbreviation, load_bigbio_dataset
from bioel.utils.dataset_consts import CUIS_TO_REMAP, CUIS_TO_EXCLUDE, VALIDATION_DOCUMENT_IDS
from bioel.ontology import BiomedicalOntology
from sklearn.feature_extraction.text import TfidfVectorizer

from bioel.ontology import BiomedicalOntology
from typing import List


def cuis_to_aliases(ontology: BiomedicalOntology , save_dir: str, dataset_name: str):
    """
    Creates the .txt file that maps the Cuis to the aliases and canonical name for each entities 
    from the provided ontology.

    Parameters
    ----------
    ontology_dir: str, required.
        The path where the ontology is stored.
    save_dir : str, required.
        The path where the mapping file "{dataset_name}_aliases.txt" will be stored
    dataset_name : str, required.
        The name of the dataset for which the mapping will be created.
    """
    # Create the directory if it doesn't exist
    os.makedirs(save_dir, exist_ok=True)
    import re

    file_path = os.path.join(save_dir, f"{dataset_name}_aliases.txt")
    with open(file_path, "w", encoding='utf-8') as file:
        for cui, entity in ontology.entities.items():
            file.write(f"{cui}||{entity.name}\n")                               
            if len(entity.aliases) != 0 :
                if isinstance(entity.aliases, list):
                    for word in entity.aliases:
                        if word is not None:
                            #print(word)
                            word = word.strip()
                            file.write(f"{cui}||{word}\n")
                else:
                    #words = entity.aliases.split('; , ')
                    words = re.split('[;|]', entity.aliases)
                    for word in words:
                        word = word.strip()
                        file.write(f"{cui}||{word}\n")
            if entity.equivalant_cuis :
                for eqcui in entity.equivalant_cuis:
                    if eqcui != entity.cui:
                        file.write(f"{eqcui}||{entity.name}\n")
                        if entity.aliases :
                            words = re.split('[;|]', entity.aliases)
                            for word in words:
                                word = word.strip()
                                file.write(f"{eqcui}||{word}\n")
    
    return(file_path)





if __name__ == "__main__":
    # data_preprocess(
    #     dataset_name = "bc5cdr", 
    #     save_dir = "data2/", 
    #     ontology_dir = "Y:/mitchell/entity-linking/2017AA/META/", 
    #     ontology_name = "mesh", 
    #     resolve_abbrevs=False)
    # cuis_to_aliases(ontology = BiomedicalOntology.load_medic(filepath="/mitchell/entity-linking/kbs/medic.tsv", name="medic") , save_dir = "data_bis/", dataset_name = "ncbi_disease")
    cuis_to_aliases(ontology = BiomedicalOntology.load_entrez(filepath="/mitchell/entity-linking/el-robustness-comparison/data/gene_info.tsv", dataset="gnormplus") , 
                    save_dir = "data_bis/", dataset_name = "gnormplus")
    cuis_to_aliases(ontology = BiomedicalOntology.load_entrez(filepath="/mitchell/entity-linking/el-robustness-comparison/data/gene_info.tsv", dataset="nlm_gene") , 
                    save_dir = "data_bis/", dataset_name = "nlm_gene")
    # cuis_to_aliases(ontology = BiomedicalOntology.load_mesh(filepath="/mitchell/entity-linking/2017AA/META/", name="mesh"), 
    #                 save_dir = "data_bis/", dataset_name = "bc5cdr")
    # cuis_to_aliases(ontology = BiomedicalOntology.load_mesh(filepath="/mitchell/entity-linking/2017AA/META/", name="mesh"), 
    #                 save_dir = "data_bis/", dataset_name = "nlm_chem")

    # cuis_to_aliases(ontology = BiomedicalOntology.load_umls(filepath="/mitchell/entity-linking/2017AA/META/", name="umls"), 
    #                 save_dir = "data_bis/", dataset_name = "medmentions_full")
    
    # cuis_to_aliases(ontology = BiomedicalOntology.load_umls(filepath="/mitchell/entity-linking/2017AA/META/", name="umls"), 
    #                 save_dir = "data_bis/", dataset_name = "medmentions_st21pv")
    
    #onto = BiomedicalOntology.load_entrez(filepath="/mitchell/entity-linking/el-robustness-comparison/data/gene_info.tsv", dataset="gnormplus")


import subprocess
import os
import logging

# Set up logging
logging.basicConfig(
    level=logging.INFO, format="%(asctime)s - %(levelname)s - %(message)s"
)

from bioel.utils.solve_abbreviation.abbreviations_utils import (
    extract_document_text,
    process_abbreviations,
)


# Function to run shell commands
def run_command(command, cwd=None):
    """Executes a shell command in a subprocess, handling exceptions."""
    try:
        subprocess.run(command, shell=True, text=True, cwd=cwd, check=True)
        logging.info(f"Successfully executed command: {command}")
    except subprocess.CalledProcessError as e:
        logging.error(f"Command failed: {command}. Error: {e}")
        raise


def clone_or_update_repo(repo_url, directory_name):
    """Clone a repo if it doesn't exist, or pull updates if it does."""
    if not os.path.exists(directory_name):
        run_command(f"git clone {repo_url} {directory_name}")
        print(f"Cloned {directory_name} from {repo_url}")
    else:
        run_command("git pull", cwd=directory_name)
        print(
            f"Updated {directory_name} with the latest changes from the remote repository."
        )


def create_abbrev(output_dir, all_dataset):
    """
    Create abbreviations.json that will contains abbreviations for each document in the specified datas
    ---------
    Parameter
    - output_dir : Path to directory where to save "abbreviations.json" file
    - all_dataset : Datasets for which you want to find abbreviations
    Ex : all_dataset = ["medmentions_full", "bc5cdr", "gnormplus", "ncbi_disease", "nlmchem", "nlm_gene"]
    """

    # Install the packages
    clone_or_update_repo("https://github.com/davidkartchner/Ab3P.git", "Ab3P")
    clone_or_update_repo("https://github.com/ncbi-nlp/NCBITextLib.git", "NCBITextLib")
    # # Initialize and update submodules
    # run_command("git submodule init")
    # run_command("git submodule update")

    # Install NCBITextLib
    run_command("make", cwd="NCBITextLib/lib")

    # Build Ab3P and run tests
    run_command("make", cwd="Ab3P")
    run_command("make test", cwd="Ab3P")

    # Get text of all articles in benchmark
    extract_document_text(output_dir, all_dataset)

    all_articles_path = os.path.join(output_dir, "all_article_text.txt")
    raw_abbreviations_path = os.path.join(output_dir, "raw_abbreviations.txt")
    # Run Ab3P to detect abbreviations
    run_command(
        f"./identify_abbr {all_articles_path} > {raw_abbreviations_path}",
        cwd="Ab3P",
    )

    # Extract abbreviation dictionary from processed file
    process_abbreviations(output_dir, all_dataset)


if __name__ == "__main__":
    all_dataset = [
        "medmentions_full",
        "bc5cdr",
        "gnormplus",
        "ncbi_disease",
        "nlmchem",
        "nlm_gene",
    ]
    output_dir = ""
    create_abbrev(output_dir, all_dataset)
